typedef struct address_s {
	uint32_t id;            /* only when unicast (->bcast==0) */
	unsigned valid:1;
	unsigned bcast:1;       /* send to all (slave) clients */
	unsigned ev_open:1;     /* annunce new client open() */
	unsigned ev_close:1;    /* annunce client close() */
} address_t;

/* like strtol(), but terminates after *len, and not at \0 */
static uint32_t parse_hex(const char *src, char **end, int *len)
{
	uint32_t acc = 0;
	for(;*len > 0; (*len)--,src++) {
		unsigned int dig;
		if ((*src >= '0') && (*src <= '9')) dig = *src - '0';
		else if ((*src >= 'a') && (*src <= 'f')) dig = *src - 'a' + 10;
		else if ((*src >= 'A') && (*src <= 'F')) dig = *src - 'A' + 10;
		else {
			*end = (char *)src;
			return acc;
		}
		acc <<= 4;
		acc |= dig;
	}

	/* ran out of *len */
	*end = NULL;
	return 0xffffffffUL;
}

/* read address from src, build dst and return the first non-address byte of
   src (or NULL on error) */
static inline char *addr_parse(address_t *dst, const char *src, int *len)
{
	char *end;

	dst->valid = 0;

	if (*len < 3)
		return NULL;

	if (*src != '{')
		return NULL;
	src++;
	(*len)--;

	if (*src == '*') {
		src++;
		(*len)--;
		end = (char *)src;
		dst->bcast = 1;
		dst->id = 0xffffffffUL;
	}
	else {
		dst->bcast = 0;
		dst->id = parse_hex(src, &end, len);
	}

	if ((end == NULL) || (*end != '}'))
		return NULL;
	end++;
	(*len)--;

	dst->ev_close = dst->ev_open = 0;
	dst->valid = 1;
	return end;
}

/* Append address prefix {src} to dst */
static inline void addr_append(gds_t *dst, uint32_t src)
{
	char tmp[128];
	sprintf(tmp, "%x", src);

	gds_append(dst, '{');
	gds_append_str(dst, tmp);
	gds_append(dst, '}');
}
